#define OEMRESOURCE

#include "custctls.h"

// TDropDownButton implementation
//
DEFINE_RESPONSE_TABLE1(TDropDownButton, TButton)
  EV_WM_LBUTTONDOWN,
END_RESPONSE_TABLE;

TDropDownButton::TDropDownButton(TWindow* parent, int resId, TModule* module)
:
 TButton(parent, resId, "", 0, 0, 0, 0, false , module)
{
  Attr.Style = BS_BITMAP | BS_CENTER | BS_NOTIFY | BS_VCENTER | WS_CHILD;

  bitmap = new TBitmap(0, MAKEINTRESOURCE(OBM_COMBO));
}

TDropDownButton::~TDropDownButton()
{
  delete bitmap;
}

void
TDropDownButton::SetupWindow()
{
  TButton::SetupWindow();

  SendMessage(BM_SETIMAGE, IMAGE_BITMAP, (long)(HBITMAP)*bitmap);
}

void
TDropDownButton::EvLButtonDown(uint modKeys, TPoint& point)
{
  SetFocus();

  TButton::EvLButtonDown(modKeys, point);
}

// TOwnerDropButton implementation
//
DEFINE_RESPONSE_TABLE1(TOwnerDropButton, TButton)
  EV_WM_LBUTTONDOWN,
END_RESPONSE_TABLE;

TOwnerDropButton::TOwnerDropButton(TWindow* parent, int resId, TModule* module)
:
 TButton(parent, resId, "...", 0, 0, 0, 0, false, module)
{
  Attr.Style = BS_CENTER | BS_NOTIFY | BS_VCENTER | WS_CHILD;
}

void
TOwnerDropButton::EvLButtonDown(uint modKeys, TPoint& point)
{
  SetFocus();

  TButton::EvLButtonDown(modKeys, point);
}

// TSubRichEdit implementation
//
DEFINE_RESPONSE_TABLE1(TSubRichEdit, TEdit)
  EV_WM_KILLFOCUS,
END_RESPONSE_TABLE;

TSubRichEdit::TSubRichEdit(TWindow* parent, int Id, const char* text, int x,
                           int y, int w, int h, const char* fileName,
                           TModule* module)
:
 TRichEdit(parent, Id, text, x, y, w, h, fileName, module),
 allowEdit(true)
{
  Attr.Style = ES_NOHIDESEL | ES_AUTOHSCROLL | WS_CHILD | WS_TABSTOP;
}

bool
TSubRichEdit::PreProcessMsg(MSG& msg)
{
  uint parentMsg = 0;

  switch (msg.message) {
    case WM_CHAR:
    case WM_COPY:
    case WM_CUT:
    case WM_KEYUP:
    case WM_PASTE:
      if (!allowEdit)
        return true;
      break;

    case WM_KEYDOWN:
      switch (msg.wParam) {
        case VK_RETURN:
          parentMsg = SN_RETURN;
          break;

        case VK_ESCAPE:
          parentMsg = SN_ESCAPE;
          break;

        case VK_DOWN:
          parentMsg = SN_DOWN;
          break;

        case VK_UP:
          parentMsg = SN_UP;
          break;

        default:
          if (!allowEdit)
            return true;
          break;
      }
  }

  if (parentMsg) {
    Parent->SendNotification(Attr.Id, parentMsg, *this);
    return true;
  }

  return TRichEdit::PreProcessMsg(msg);
}

void
TSubRichEdit::EvKillFocus(HWND hWndGetFocus)
{
  Parent->SendNotification(Attr.Id, SN_KILLFOCUS, *this);

  TRichEdit::EvKillFocus(hWndGetFocus);
}

// TSubListBox implementation
//
DEFINE_RESPONSE_TABLE1(TSubListBox, TListBox)
  EV_WM_KILLFOCUS,
END_RESPONSE_TABLE;

TSubListBox::TSubListBox(TWindow* parent, int Id, int x, int y, int w, int h,
                         TModule* module)
:
 TListBox(parent, Id, x, y, w, h, module)
{
  Attr.Style = LBS_HASSTRINGS | LBS_NOTIFY | LBS_WANTKEYBOARDINPUT | WS_BORDER |
               WS_CHILD | WS_VSCROLL;
}

bool
TSubListBox::PreProcessMsg(MSG& msg)
{
  uint parentMsg = 0;

  if (msg.message == WM_KEYDOWN)
    switch (msg.wParam) {
      case VK_RETURN:
        parentMsg = SN_RETURN;
        break;

      case VK_ESCAPE:
        parentMsg = SN_ESCAPE;
        break;
    }

  if (parentMsg) {
    Parent->SendNotification(Attr.Id, parentMsg, *this);
    return true;
  }

  return TListBox::PreProcessMsg(msg);
}

void
TSubListBox::EvKillFocus(HWND hWndGetFocus)
{
  Parent->SendNotification(Attr.Id, SN_KILLFOCUS, *this);

  TListBox::EvKillFocus(hWndGetFocus);
}

