#if !defined(TOOL_CUSTCTLS_H)
#define TOOL_CUSTCTLS_H

#if !defined(OWL_BUTTON_H)
# include <owl/button.h>
#endif
#if !defined(OWL_LISTBOX_H)
# include <owl/listbox.h>
#endif
#if !defined(OWL_RICHEDIT_H)
# include <owl/richedit.h>
#endif


// Subclassed control notification messages
//
#define SN_RETURN    WM_USER + 1001
#define SN_ESCAPE    WM_USER + 1002
#define SN_DOWN      WM_USER + 1003
#define SN_UP        WM_USER + 1004
#define SN_LEFT      WM_USER + 1005
#define SN_RIGHT     WM_USER + 1006
#define SN_KILLFOCUS WM_USER + 1007

//
// Class TDropDownButton
// ~~~~~ ~~~~~~~~~~~~~~~
// TDropDownButton is a button that displays a
// downward arrow implying that a drop down
// list will be shown if pressed
//
class TDropDownButton : public TButton {
  public:
    TDropDownButton(TWindow* parent, int resId, TModule* module = 0);
   ~TDropDownButton();

    void SetupWindow();

  protected:
    void EvLButtonDown(uint modKeys, TPoint& point);

  private:
    TBitmap* bitmap;

  DECLARE_RESPONSE_TABLE(TDropDownButton);
};

//
// Class TOwnerDropButton
// ~~~~~ ~~~~~~~~~~~~~~~~
// TOwnerDropButton is a button that displays
// ellipsis implying that a subsequent owner
// drop dialog will be shown if pressed
//
class TOwnerDropButton : public TButton {
  public:
    TOwnerDropButton(TWindow* parent, int resId, TModule* module = 0);

  protected:
    void EvLButtonDown(uint modKeys, TPoint& point);

  DECLARE_RESPONSE_TABLE(TOwnerDropButton);
};

//
// Class TSubRichEdit
// ~~~~~ ~~~~~~~~~~~~
// TSubRichEdit is subclassed from TRichEdit in order to
// customize its behavior
//
class TSubRichEdit : public TRichEdit {
  public:
    TSubRichEdit(TWindow* parent, int Id, const char* text, int x, int y, int w,
                 int h, const char* fileName = 0, TModule* module = 0);

    void AllowEdit(bool allow);

    // Override virtuals from TWindow
    //
    bool CanClose();

    bool PreProcessMsg(MSG& msg);

  protected:
    void EvKillFocus(HWND hWndGetFocus);

  private:
    bool allowEdit;

  DECLARE_RESPONSE_TABLE(TSubRichEdit);
};

//
// Class TSubListBox
// ~~~~~ ~~~~~~~~~~~
// TSubListBox is subclassed from TListBox in order to
// customize its behavior
//
class TSubListBox : public TListBox {
  public:
    TSubListBox(TWindow* parent, int Id, int x, int y, int w, int h,
                TModule* module = 0);

    bool PreProcessMsg(MSG& msg);

  protected:
    void EvKillFocus(HWND hWndGetFocus);

  DECLARE_RESPONSE_TABLE(TSubListBox);
};

// TSubRichEdit inline implementation
//
inline void
TSubRichEdit::AllowEdit(bool allow)
{
  allowEdit = allow;
}

inline bool
TSubRichEdit::CanClose()
{
  // Always let to close
  //
  return true;
}

#endif  // TOOL_CUSTCTLS_H

