#include <owl/applicat.h>
#include <owl/chooseco.h>
#include <owl/choosefo.h>
#include <owl/framewin.h>
#include <owl/opensave.h>

#include "propbox.h"

#include "demo.rc"

TChooseColorDialog::TData colorData;;
TChooseFontDialog::TData  fontData;
TOpenSaveDialog::TData    fileData(OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
                                   "All Files (*.*)|*.*|", 0, 0, "*");

class TDemoPropBox : public TPropertyBox {
  public:
    TDemoPropBox(TWindow* parent);
   ~TDemoPropBox();

    void Destroy(int retVal = 0);

  protected:
    void SetupWindow();

    TResult EvNotifyPropChange (TPropNotify& notifyInfo);
    TResult EvNotifyOwnerDrop  (TPropNotify& notifyInfo);
    TResult EvNotifyPressButton(TPropNotify& notifyInfo);
};

inline
TDemoPropBox::TDemoPropBox(TWindow* parent)
:
 TPropertyBox(parent)
{
}

inline
TDemoPropBox::~TDemoPropBox()
{
}

void
TDemoPropBox::Destroy(int retVal)
{
  for (int i = 0; i < listBox->GetCount(); i++) {
    TPropertyItem* item = listBox->GetProperty(i);

    if (item)
      MessageBox(item->GetValue(), item->GetProperty());
  }

  TPropertyBox::Destroy(retVal);
}

void
TDemoPropBox::SetupWindow()
{
  TPropertyBox::SetupWindow();

  listBox->AddProperty(new TPropertyItem(listBox, "Window Caption",
                                         "Try to change this text..."));

  TRect rcList;
  GetClientRect(rcList);

  listBox->AddProperty(new TPropertyItem(listBox, "Width",  rcList.Width(),
                                         PROP_READONLY));
  listBox->AddProperty(new TPropertyItem(listBox, "Height", rcList.Height(),
                                         PROP_EDIT));

  // This item will not be inserted because a property
  // with the same name has already been defined
  //
  listBox->AddProperty(new TPropertyItem(listBox, "Height", rcList.Height(),
                                         PROP_EDIT));

  // Color
  //
  static TColor custColors[16] =
  {
    0x010101L, 0x101010L, 0x202020L, 0x303030L,
    0x404040L, 0x505050L, 0x606060L, 0x707070L,
    0x808080L, 0x909090L, 0xA0A0A0L, 0xB0B0B0L,
    0xC0C0C0L, 0xD0D0D0L, 0xE0E0E0L, 0xF0F0F0L
  };

  colorData.Flags      = CC_RGBINIT;
  colorData.Color      = TColor::LtRed;
  colorData.CustColors = custColors;

  listBox->AddProperty(new TColorPropItem(listBox, "Color", &colorData));

  // File name
  //
  listBox->AddProperty(new TFileNamePropItem(listBox, "FileName", &fileData,
                                             false));

  //list
  //
  TListPropItem *player = new TListPropItem(listBox, "Player",
                                            "Sebastiano Rossi");

  player->AddString("Mauro Tassotti");
  player->AddString("Paolo Maldini");
  player->AddString("Carlo Ancelotti");
  player->AddString("Filippo Galli");
  player->AddString("Franco Baresi");
  player->AddString("Roberto Donadoni");
  player->AddString("Frank Rijkaard");
  player->AddString("Marco Van Basten");
  player->AddString("Ruud Gullit");
  player->AddString("Alberigo Evani");

  listBox->AddProperty(player);

  // Font
  //
  fontData.DC = TWindowDC(*this);
  fontData.Flags = CF_BOTH | CF_EFFECTS | CF_INITTOLOGFONTSTRUCT;
  GetObject(GetWindowFont(), sizeof(LOGFONT), &fontData.LogFont);

  listBox->AddProperty(new TFontPropItem(listBox, "Font", &fontData, false));
}

TResult
TDemoPropBox::EvNotifyPropChange(TPropNotify& notifyInfo)
{
  if (!strcmp(notifyInfo.pItem->GetProperty(), "Window Caption"))
    SetWindowText(notifyInfo.pItem->GetValue());
  return 0;
}

TResult
TDemoPropBox::EvNotifyOwnerDrop(TPropNotify& /* notifyInfo */)
{
  return 0;
}

TResult
TDemoPropBox::EvNotifyPressButton(TPropNotify& /* notifyInfo */)
{
  return 0;
}

class TDemoWin : public TWindow {
  public:
    TDemoWin(TWindow* parent);

  protected:
    void CmTestPropBox();

  DECLARE_RESPONSE_TABLE(TDemoWin);
};

DEFINE_RESPONSE_TABLE1(TDemoWin, TWindow)
  EV_COMMAND(CM_TEST_PROPBOX, CmTestPropBox),
END_RESPONSE_TABLE;

inline
TDemoWin::TDemoWin(TWindow* parent)
:
 TWindow(parent)
{
}

void
TDemoWin::CmTestPropBox()
{
  TDemoPropBox* propBox = new TDemoPropBox(this);
  propBox->Create();

  propBox->SetFlag(wfDeleteOnClose);
}

class TDemoApp : public TApplication {
  public:
    TDemoApp(const char far* name);

    void InitMainWindow();
};

inline
TDemoApp::TDemoApp(const char far* name)
:
 TApplication(name)
{
}

void
TDemoApp::InitMainWindow()
{
  TFrameWindow* frame = new TFrameWindow(0, "Property Box Demo Application",
                                         new TDemoWin(0));
  frame->AssignMenu(IDM_MENU);

  SetMainWindow(frame);
}

int
OwlMain(int /* argc */, char* /* argv */ [])
{
  return TDemoApp("Demo").Run();
}

