#include "propbox.h"

#include "propbox.rc"

DEFINE_RESPONSE_TABLE1(TPropertyBox, TDialog)
  EV_WM_SIZE,
  EV_HDN_TRACK   (IDC_HEADER, EvTrack),
  EV_HDN_ENDTRACK(IDC_HEADER, EvEndTrack),
END_RESPONSE_TABLE;

TPropertyBox::TPropertyBox(TWindow* parent)
:
 TDialog(parent, IDD_PROPBOX, ::Module)
{
  listBox = new TPropListBox(this, IDC_LISTBOX);

  header = new TColumnHeader(this, IDC_HEADER);

  propHdr  = new THdrItem("Property");
  valueHdr = new THdrItem("Value");
}

TPropertyBox::~TPropertyBox()
{
  delete listBox;

  delete header;

  delete propHdr;
  delete valueHdr;
}

TResult
TPropertyBox::EvNotify(uint id, TNotify& notifyInfo)
{
  switch (notifyInfo.code) {
    case PBN_PROPCHANGE:
      return EvNotifyPropChange(static_cast<TPropNotify&>(notifyInfo));

    case PBN_OWNERDROP:
      return EvNotifyOwnerDrop(static_cast<TPropNotify&>(notifyInfo));

    case PBN_PRESSBUTTON:
      return EvNotifyPressButton(static_cast<TPropNotify&>(notifyInfo));
  }

  return TDialog::EvNotify(id, notifyInfo);
}

void
TPropertyBox::SetupWindow()
{
  TDialog::SetupWindow();

  // Adjust the position of Property Box
  //
  TRect rect;
  GetWindowRect(rect);

  rect.left =  ::GetSystemMetrics(SM_CXSCREEN) * 9 / 10 - rect.Width();
  rect.top  = (::GetSystemMetrics(SM_CYSCREEN) - rect.Height()) / 2;

  SetWindowPos(0, rect, SWP_NOACTIVATE | SWP_NOSIZE);

  // Add headers
  //
  propHdr ->SetMask(HDI_WIDTH | HDI_TEXT);
  valueHdr->SetMask(HDI_WIDTH | HDI_TEXT);

  header->Add(*propHdr);
  header->Add(*valueHdr);

  // Adjust the position of headers and client list box
  //
  Resize(true);
}

void
TPropertyBox::EvSize(uint sizeType, TSize& size)
{
  TDialog::EvSize(sizeType, size);

  Resize();
}

bool
TPropertyBox::EvTrack(THdrNotify& nmHdr)
{
  if (nmHdr.iItem == 1)
    return true;  // Prevent the value header from being tracked
  else
    return false;
}

void
TPropertyBox::EvEndTrack(THdrNotify& nmHdr)
{
  if (nmHdr.pitem->mask & HDI_WIDTH) {
    int width = nmHdr.pitem->cxy;

    listBox->SetPropertyWidth(width);

    TRect rect;
    GetClientRect(rect);

    valueHdr->SetWidth(rect.Width() - width);

    header->SetItem(*valueHdr, 1);

    header->Layout();
  }
}

TResult
TPropertyBox::EvNotifyPropChange(TPropNotify& /* notifyInfo */)
{
  return 0;
}

TResult
TPropertyBox::EvNotifyOwnerDrop(TPropNotify& /* notifyInfo */)
{
  return 0;
}

TResult
TPropertyBox::EvNotifyPressButton(TPropNotify& /* notifyInfo */)
{
  return 0;
}

void
TPropertyBox::Resize(bool firstTime)
{
  TRect liRect;
  GetClientRect(liRect);

  TRect hdrRect;
  header->GetClientRect(hdrRect);

  // This is to prevent the bottom of the header from
  // moving down after appearing for the first time
  //
  if (firstTime)
    hdrBottomPos = hdrRect.bottom - 1;

  hdrRect.bottom = hdrBottomPos;

  liRect.top += hdrRect.Height() + 1;
  listBox->MoveWindow(liRect, true);

  hdrRect.right = liRect.right;
  header->MoveWindow(hdrRect, true);

  int liWidth = liRect.Width();

  int width;
  if (firstTime) {
    width = liWidth / 2;

    listBox->SetPropertyWidth(width);

    propHdr->SetWidth(width);

    header->SetItem(*propHdr, 0);
  }
  else
    width = listBox->GetPropertyWidth();

  valueHdr->SetWidth(liWidth - width);

  header->SetItem(*valueHdr, 1);

  header->Layout();
}

