#include <stdio.h>

#include "propbox.h"
#include "propitem.h"
#include "proplist.h"

// TPropertyItem implementation
//
TPropertyItem::TPropertyItem(TPropListBox* propListBox, char* property,
                             char* value, int flags)
:
 parent(propListBox), listBox(0), propName(property),
 propValue(value), itemFlags(flags), itemIndex(-1)
{
}

TPropertyItem::TPropertyItem(TPropListBox* propListBox, char* property,
                             int value, int flags)
:
 parent(propListBox), listBox(0), propName(property),
 itemFlags(flags), itemIndex(-1)
{
  char str[64];
  wsprintf(str, "%d", value);

  propValue = str;
}

void
TPropertyItem::SetValue(int value)
{
  char str[64];
  wsprintf(str, "%d", value);

  propValue = str;
}

// TColorPropItem implementation
//
TColorPropItem::TColorPropItem(TPropListBox* propListBox, char* property,
                               TChooseColorDialog::TData* data, bool edit)
:
 TPropertyItem(propListBox, property, "",
               edit ? PROP_OWNERDROP | PROP_EDIT : PROP_OWNERDROP),
 colorData(data)
{
  char str[32];
  sprintf(str, "%#08x", colorData->Color.GetValue());

  SetValue(str);
}

int
TColorPropItem::Activate()
{
  if (TChooseColorDialog(parent, *colorData).Execute() == IDOK) {
    char str[32];
    sprintf(str, "%#08x", colorData->Color.GetValue());

    SetValue(str);

    parent->EvSelChange();

    return IDOK;
  }

  return 0;
}

// TFileNamePropItem implementation
//
TFileNamePropItem::TFileNamePropItem(TPropListBox* listBox, char* property,
                                     TOpenSaveDialog::TData* data, bool edit)
:
 TPropertyItem(listBox, property, data->FileName,
               edit ? PROP_OWNERDROP | PROP_EDIT : PROP_OWNERDROP),
 fileData(data)
{
}

int
TFileNamePropItem::Activate()
{
  if (TFileOpenDialog(parent, *fileData).Execute() == IDOK) {
    SetValue(fileData->FileName);

    parent->EvSelChange();

    return IDOK;
  }

  return 0;
}

// TFontPropItem implementation
//
TFontPropItem::TFontPropItem(TPropListBox* listBox, char* property,
                             TChooseFontDialog::TData* data, bool edit)
:
 TPropertyItem(listBox, property, data->LogFont.lfFaceName,
               edit ? PROP_OWNERDROP | PROP_EDIT : PROP_OWNERDROP),
 fontData(data)
{
}

int
TFontPropItem::Activate()
{
  if (TChooseFontDialog(parent, *fontData).Execute() == IDOK) {
    SetValue(fontData->LogFont.lfFaceName);

    parent->EvSelChange();

    return IDOK;
  }

  return 0;
}

// TListPropItem implementation
//
TListPropItem::TListPropItem(TPropListBox* propListBox, char* property,
                             char* value)
:
 TPropertyItem(propListBox, property, value, PROP_DROPLIST | PROP_GETCHANGE)
{
  listBox = new TSubListBox(parent, PB_LISTBOX_ID, 0, 0, 0, 0);

  listBox->Create();

  AddString(GetValue());
}

TListPropItem::~TListPropItem()
{
  delete listBox;
}

