#if !defined(TOOL_PROPITEM_H)
#define TOOL_PROPITEM_H

#if !defined(OWL_CHOOSECO_H)
# include <owl/chooseco.h>
#endif
#if !defined(OWL_CHOOSEFO_H)
# include <owl/choosefo.h>
#endif
#if !defined(OWL_LISTBOX_H)
# include <owl/listbox.h>
#endif
#if !defined(OWL_OPENSAVE_H)
# include <owl/opensave.h>
#endif
#if !defined(__CSTRING_H)
# include <cstring.h>
#endif
#if !defined(TOOL_CUSTCTLS_H)
# include "custctls.h"
#endif

// List box ID
//
#define PB_LISTBOX_ID 1206

// Forward declarations
//
class TPropListBox;

// Flags for individual properties
//
enum PropertyFlags {
  PROP_READONLY  = 0x0001,  // Read only property
  PROP_EDIT      = 0x0002,  // Edit control
  PROP_DROPLIST  = 0x0004,  // Drop down list of values
  PROP_OWNERDROP = 0x0008,  // Drop down, owner handles
  PROP_GETCHANGE = 0x0010,  // Notify of changes
  PROP_INTEGER   = 0x0020   // Integer type
};

//
// Class TPropertyItem
// ~~~~~ ~~~~~~~~~~~~~
// TPropertyItem represents a generic property in a TPropertyBox
//
class TPropertyItem {
  public:
    TPropertyItem(TPropListBox* propListBox, char* property, char* value,
                  int flags = PROP_EDIT | PROP_GETCHANGE);
    TPropertyItem(TPropListBox* propListBox, char* property, int   value,
                  int flags = PROP_EDIT | PROP_GETCHANGE);

    // Accessors
    //
    TSubListBox* GetListBox() const;

    const char* GetProperty() const;
    const char* GetValue   () const;

    int GetFlags() const;

    int GetIndex() const;

    // Mutators
    //
    virtual int Activate();

    void SetProperty(const char* property);
    void SetValue   (const char* value   );
    void SetValue   (int         value   );

    void SetIndex(int index);

  protected:
    TPropListBox* parent;

    TSubListBox* listBox;  // Used by derived classes

  private:
    string propName;
    string propValue;

    int itemFlags;

    int itemIndex;
};

//
// Class TColorPropItem
// ~~~~~ ~~~~~~~~~~~~~~
// TColorPropItem represents a color property in a TPropertyBox
//
class TColorPropItem : public TPropertyItem {
  public:
    TColorPropItem(TPropListBox* propListBox, char* property,
                   TChooseColorDialog::TData* data, bool edit = false);

    int Activate();

  private:
    TChooseColorDialog::TData* colorData;
};

//
// Class TFileNamePropItem
// ~~~~~ ~~~~~~~~~~~~~~~~~
// TFileNamePropItem represents a file name property in a TPropertyBox
//
class TFileNamePropItem : public TPropertyItem {
  public:
    TFileNamePropItem(TPropListBox* propListBox, char* property,
                      TOpenSaveDialog::TData* data, bool edit = false);

    int Activate();

  private:
    TOpenSaveDialog::TData* fileData;
};

//
// Class TFontPropItem
// ~~~~~ ~~~~~~~~~~~~~
// TFontPropItem represents a font property in a TPropertyBox
//
class TFontPropItem : public TPropertyItem {
  public:
    TFontPropItem(TPropListBox* propListBox, char* property,
                  TChooseFontDialog::TData* data, bool edit = false);

    int Activate();

  private:
    TChooseFontDialog::TData* fontData;
};

//
// Class TListPropItem
// ~~~~~ ~~~~~~~~~~~~~
// TListPropItem represents a list property in a TPropertyBox
//
class TListPropItem : public TPropertyItem {
  public:
    TListPropItem(TPropListBox* propListBox, char* property, char* value);
   ~TListPropItem();

    int AddString(const char* str);
};

// TPropertyItem inline implementaion
//
inline TSubListBox*
TPropertyItem::GetListBox() const
{
  return listBox;
}

inline const char*
TPropertyItem::GetProperty() const
{
  return propName.c_str();
}

inline const char*
TPropertyItem::GetValue() const
{
  return propValue.c_str();
}

inline int
TPropertyItem::GetFlags() const
{
  return itemFlags;
}

inline int
TPropertyItem::GetIndex() const
{
  return itemIndex;
}

inline int
TPropertyItem::Activate()
{
  return 0;
}

inline void
TPropertyItem::SetProperty(const char* property)
{
  if (property)
    propName = property;
}

inline void
TPropertyItem::SetValue(const char* value)
{
  if (value)
    propValue = value;
}

inline void
TPropertyItem::SetIndex(int index)
{
  itemIndex = index;
}

// TListPropItem inline implementation
//
inline int
TListPropItem::AddString(const char* str)
{
  return listBox->AddString(str);
}

#endif  // TOOL_PROPITEM_H