#if !defined(TOOL_PROPLIST_H)
#define TOOL_PROPLIST_H

#if !defined(OWL_COMMCTRL_H)
# include <owl/commctrl.h>
#endif
#if !defined(OWL_LISTBOX_H)
# include <owl/listbox.h>
#endif
#if !defined(CLASSLIB_ASSOC_H)
# include <classlib/assoc.h>
#endif
#if !defined(CLASSLIB_DICT_H)
# include <classlib/dict.h>
#endif
#if !defined(TOOL_CUSTCTLS_H)
# include "custctls.h"
#endif
#if !defined(TOOL_PROPITEM_H)
# include "propitem.h"
#endif

// TPropListBox notification messages
//
#define PBN_PROPCHANGE  WM_USER + 1011
#define PBN_OWNERDROP   WM_USER + 1012
#define PBN_PRESSBUTTON WM_USER + 1013

// Enumeration values to force ToggleList to show
// hide or toggle the list between two states
//
enum ListState {
  neutral,
  show,
  hide
};

// The global function, HashValue, is required for the
// operation of the TDictionaryAsHashTable container
//
inline unsigned
HashValue(const string& str)
{
  return str.hash();
}

typedef TDIAssociation<string, TPropertyItem>           TStrToItemAssoc;
typedef TDictionaryAsHashTable<TStrToItemAssoc>         TStrToItemMap;
typedef TDictionaryAsHashTableIterator<TStrToItemAssoc> TStrToItemMapIter;

//
// Class TPropNotify
// ~~~~~ ~~~~~~~~~~~
// TPropNotify gives information about a specific
// property item being changed
//
class TPropNotify : public TNotify {
  public:
    TPropNotify(HWND ctl, uint id, uint code, TPropertyItem* item, TRect& rect);

    TPropertyItem* pItem;  // Property item

    TRect ownerDropRect;   // Suggested rect for drop down list box
};

//
// Class TPropListBox
// ~~~~~ ~~~~~~~~~~~~
// TPropListBox is the main class that manages the
// task of displaying property items and maintains
// the relationship between properties and their
// respective values
//
class TPropListBox : public TListBox {
  public:
    TPropListBox(TWindow* parent, int resourceId, TModule* module = 0);
   ~TPropListBox();

    // Accessors
    //
    TPropertyItem* GetProperty(int         index   );
    TPropertyItem* GetProperty(const char* property);

    int GetPropertyWidth() const;

    // Mutators
    //
    int AddProperty(TPropertyItem* item    );
    int DelProperty(const char*    property);

    bool SetValue(TPropertyItem* item, const char* value);

    void SetPropertyWidth(int width);

    void EvSelChange();

    // Override virtuals from TWindow
    //
    TResult EvCommand(uint id, THandle hWndCtl, uint notifyCode);

    bool PreProcessMsg(MSG& msg);

  protected:
    void SetupWindow();

    void EvDrawItem(uint ctrlId, DRAWITEMSTRUCT& drawInfo);
    void EvSetFocus(THandle hWndLostFocus);
    void EvSize    (uint sizeType, TSize& size);
    void EvVScroll (uint scrollCode, uint thumbPos, HWND hWndCtl);

    // Notify the application of changes being made to property items
    //
    bool SendNotification(uint code, TPropertyItem* item, TRect& rect);

    bool SendPropChange (TPropertyItem* item);
    bool SendPressButton(TPropertyItem* item);
    bool SendOwnerDrop  (TPropertyItem* item, TRect& rect);

    // Toggle the display of the value list for a property
    //
    void ToggleList(TPropertyItem* item, ListState state = neutral);

    // Hide the floating child windows
    //
    void HideChildren();

  private:
    // Floating child window members
    //
    TSubRichEdit*     edit;
    TDropDownButton*  button;
    TOwnerDropButton* odButton;
    TSubListBox*      listBox;

    // Map of property names to values
    //
    TStrToItemMap strToItemMap;

    // Color data
    //
    TColor textColor;
    TColor selTextColor;
    TColor readOnlyTextColor;
    TColor gridColor;
    TColor propertyBackColor;
    TColor selPropertyBackColor;
    TColor valueBackColor;

    // Pen used to draw grid lines
    //
    TPen* gridPen;

    // Rectangle surrounding the value part
    //
    TRect valueRect;

    // Width of the property part
    //
    int propertyWidth;

  DECLARE_RESPONSE_TABLE(TPropListBox);
};

// TPropListBox inline implementation
//
inline int
TPropListBox::GetPropertyWidth() const
{
  return propertyWidth;
}

inline bool
TPropListBox::SendPropChange(TPropertyItem* item)
{
  return SendNotification(PBN_PROPCHANGE, item, TRect(0, 0, 0, 0));
}

inline bool
TPropListBox::SendPressButton(TPropertyItem* item)
{
  item->Activate();

  return SendNotification(PBN_PRESSBUTTON, item, TRect(0, 0, 0, 0));
}

inline bool
TPropListBox::SendOwnerDrop(TPropertyItem* item, TRect& rect)
{
  return SendNotification(PBN_OWNERDROP, item, rect);
}

#endif  // TOOL_PROPLIST_H
